/******************************************************************************
 *                                                                            *
 *                 S A S   I N S T A L L A T I O N   T E S T                  *
 *                                                                            *
 *      NAME: assert                                                          *
 *     TITLE: Marco for checking return codes, stopping SAS and returning     *
 *            specific values from SAS.                                       *
 *     INPUT: An integer value to be checked, possibly an expected value      *
 *            (which defaults to zero), and possibly a comparison operator    *
 *            (which defaults to "ne").                                       *
 *    OUTPUT: A %put message to the SAS log, and specific SAS return code     *
 *     USAGE:                                                                 *
 *                                                                            *
 *    %rccheck(integer<,integer>,<operator>);                                 *
 *                                                                            *
 *    The first parameter is required.  It is the actual value that you       *
 *    wish to check.  Typically, it is one of the SAS System defined          *
 *    automatic macro variables, &syserr, &sysinfo, &sysrc, &syslibrc, etc.   *
 *    However, it can be any integer you care to specify.  If the actual      *
 *    value does not equal the expected value, SAS is aborted with its        *
 *    return code set to the actual value you specified in the rccheck        *
 *    macro invocation.  If the actual value does not equal the expected      *
 *    value and the actual value is not in the range 0-254, the SAS           *
 *    return code will be set to 254.  254 is the maximum portable value      *
 *    that SAS will accept for this variable.                                 *
 *                                                                            *
 *    The second parameter is optional.  It is the expected value.  If it     *
 *    is not specified, the default is zero.                                  *
 *                                                                            *
 *                                                                            *
 *    The third paramter is optional.  It the the comparison operator.        *
 *    If it is not specified, it defaults to "ne".  If specified, it is       *
 *    used to compare the actual value to the expected value.  If the         *
 *    comparison results in a TRUE result, the abort logic is executed.       *
 *                                                                            *
 *    The rccheck macro cannot be used in a normal manner within a datastep   *
 *    because it forms a datastep of its own; thus prematurely terminating    *
 *    the other datastep.  Use the symput and symget functions to set         *
 *    return codes inside a datastep, then use the rccheck macro external     *
 *    to the datastep.                                                        *
 *                                                                            *
 *                                                                            *
 * EXAMPLES:                                                                  *
 *                                                                            *
 *    (1) Checking on a LIBNAME statement:                                    *
 *                                                                            *
 *    LIBNAME FOO V7 '.';                                                     *
 *    %rccheck(&syslibrc);                                                    *
 *                                                                            *
 *    (2) Checking on a PROC:                                                 *
 *                                                                            *
 *    PROC PRINT DATA=WORK.FOO;                                               *
 *    RUN;                                                                    *
 *    %rccheck(&syserr);                                                      *
 *                                                                            *
 *    (3) Checking closely on *some* PROCs(those that set &sysinfo):          *
 *        (This code verifies that PROC COMPARE has found two SAS             *
 *        datasets essentially equivalent, differing only by labels           *
 *        and formats.  See the PROC COMPARE doc to learn more about          *
 *        the &sysinfo macro variable.)                                       *
 *                                                                            *
 *    PROC COMPARE BASE=WORK.ONE COMPARE=WORK.TWO;                            *
 *    RUN;                                                                    *
 *    %rccheck(&sysinfo,3);                                                   *
 *                                                                            *
 *    (4) Checking on a &sysexec macro:                                       *
 *                                                                            *
 *    %sysexec ksh -c "test /u/me/mypath";                                    *
 *    %rccheck(&sysrc);                                                       *
 *                                                                            *
 *                                                                            *
 *    (5) Checking on a LIBNAME statement that should fail:                   *
 *                                                                            *
 *    LIBNAME FOO BADENGINE '.';                                              *
 *    %rccheck(&syslibrc,0,eq);                                               *
 *    In this case, the LIBNAME statement should fail and                     *
 *    &SYSLIBRC should *NOT* contain a zero.                                  *
 *                                                                            *
 ******************************************************************************/

%macro rccheck(actual, expected, operator);

   %local actual;        /* The return code value to be inspected */
   %local expected;      /* The expected value.  The value that   */
                         /* the return code will be compared to.  */
   %local operator;      /* The comparison operator to be used.   */

   /* A return code value to compare *must* be specified. */
   /* Give up right away if none was specified.           */
   %if %length(&actual) = 0 %then
   %do;
      %put Invalid use of the rccheck macro.  No return code was specified;
      data _null_; abort return 254; run;
   %end;

   /* An expected value is not required, it defaults to    */
   /* zero if not specified in the macro invocation.       */
   %if %length(&expected) = 0 %then
   %do;
      %let expected = 0;
   %end;

   /* A comparison operator is not required, it defaults to    */
   /* "ne" if not specified in the macro invocation.           */
   %if %length(&operator) = 0 %then
   %do;
      %let operator = ne;
   %end;

   /* Perform the comparison between actual and expected */
   %if &actual &operator &expected %then
   %do;
      %if &actual > 254 or &actual < 0 %then
      %do;
         %put The original return code value was &actual.;
         %put That value is not in the valid range of 0-254.;
         %put The return code value has been reset to 254.;
         %let actual=254;
      %end;
      %put Preparing to ABORT with rc of &actual;
      %put Condition &actual &operator &expected was TRUE.;
      data _null_; abort abend &actual; endsas; run;
   %end;

%mend rccheck;
